@@ -10,26 +10,23 @@ import urllib
 import urllib2
 
 WELCOME_BANNER = """
- ____      ____             __                             
-|_  _|    |_  _|           [  |                            
-  \ \  /\  / /.---.  .---.  | |  .--.   _ .--..--.  .---.  
-   \ \/  \/ // /__\\/ /'`\] | |/ .'`\ \[ `.-. .-. |/ /__\\ 
-    \  /\  / | \__.,| \__.  | || \__. | | | | | | || \__., 
-     \/  \/   '.__.''.___.'[___]'.__.' [___||__||__]'.__.' 
-                                                           
-                     _________                             
-                    |  _   _  |                            
-                    |_/ | | \_|.--.                        
-                        | |  / .'`\ \                      
-                       _| |_ | \__. |                      
-                      |_____| '.__.'                       
-                                                           
-            ________   _                   _               
-           |  __   _| (_)                 | |              
-           |_/  / /   __   _ .--.   .---. | |              
-              .'.' _ [  | [ `.-. | / /'`\]| |              
-            _/ /__/ | | |  | | | | | \__. |_|              
-           |________|[___][___||__]'.___.'(_)              
+ ____      ____      __                                     
+|_  _|    |_  _|    [  |                                    
+  \ \  /\  / /.---.  | |  .---.   .--.   _ .--..--.  .---.  
+   \ \/  \/ // /_\ \ | | / /'`\]/ .'`\ \[ `.-. .-. |/ /_\ \ 
+    \  /\  / | \__., | | | \__. | \__. | | | | | | || \__., 
+     \/  \/   '.__.'[___]'.___.' '.__.' [___||__||__]'.__.' 
+                       _________                            
+                      |  _   _  |                           
+                      |_/ | | \_|.--.                       
+                          | |  / .'`\ \                     
+                         _| |_ | \__. |                     
+              ________  |_____| '.__.'       _              
+             |  __   _| (_)                 | |             
+             |_/  / /   __   _ .--.   .---. | |             
+                .'.' _ [  | [ `.-. | / /'`\]| |             
+              _/ /__/ | | |  | | | | | \__. |_|             
+             |________|[___][___||__]'.___.'(_)             
 
 """
 
@@ -74,47 +71,46 @@ class ValidationHelpers(object):
 
 class ZincWizard(object):
     PROMPTS = {
-        "search_query": "What do you want to buy? (e.g. black cup)",
-        "select_product_name": "Here are your results. Please select a product to purchase.",
+        "search_query": "What do you want to buy? (e.g. inflatable banana)",
+        "select_product_name": "Please select a result:",
         "product_variants": "Please please enter a product URL.",
-        "product_quantity": "How many would you like to purchase? (Default: 1)",
+        "product_quantity": "How many would you like to purchase? [1]",
         "select_product_variants": "This item comes in multiple variants. Please choose an option.",
         "select_shipping_methods": "This item has multiple shipping options. Please choose an option.",
-        "security_code": "Please enter the CVV security code on your credit card.",
-        "place_order": "Would you like to place this order? (y)/n",
-        "gift": "Do you want this to be shipped as a gift? (y)/n",
+        "security_code": "Please enter your stored credit card's CVV security code.",
+        "place_order": "Would you like to place this order? [y]/n",
+        "gift": "Do you want this to be shipped as a gift? [y]/n",
         "retailer_credentials": {
-            "email": "Please enter your Amazon username (usually your email)",
+            "email": "Please enter your Amazon username (email address)",
             "password": "Please enter your Amazon password"
             },
         "shipping_address": {
-            "start_message": "\nNow we'd like to get your shipping information. Don't worry if you make a mistake, we'll ask you to verify the correctness of your address so you can retype it if you'd like.\n",
+            "start_message": "\nPlease enter your shipping information. If you make a mistake, it may be corrected at the end of this section.\n",
             "end_message": "\nYou've finished entering your shipping address!"
             },
         "address": {
-            "first_name": "Please input your first name:",
-            "last_name": "Please input your last name:",
-            "address_line1": "Please input the first line of your shipping address:",
-            "address_line2": "Please input the second line of your shipping address: (optional)",
+            "name": "Please input your full name:",
+            "address_line1": "Please input the first line of your address:",
+            "address_line2": "Please input the second line of your address: (optional)",
             "city": "Please input your city:",
             "state": "Please input your state (e.g. CA, MA, etc.):",
             "zip_code": "Please input your zip code:",
             "country": "Please input your country (e.g. US):",
-            "confirmation_message": "Is this your correct shipping address? (y)/n"
+            "confirmation_message": "Is this your correct address? [y]/n"
             },
         "billing_address" : {
-            "start_message": "\nIs your billing address the same as your shipping address? (y)/n",
+            "start_message": "\nIs your billing address the same as your shipping address? [y]/n",
             "end_message": "\nYou've finished entering you billing address!"
             },
         "credit_card": {
-            "start_message": "\nNow we'd like to get your credit card information.",
-            "number": "Please input your credit card number",
+            "start_message": "\nThe retailer requires a credit card for this purchase.",
+            "number": "Please input your credit card number (e.g. 5555555555554444)",
             "expiration_month": "Please input your credit card expiration month (e.g. 03)",
             "expiration_year": "Please input your credit card expiration year (e.g. 2017)",
-            "security_code": "Please input the CVV security code from your credit card",
+            "security_code": "Please input your card's CVV security code",
             "end_message": "\nYou've finished entering your credit card information!"
             },
-        "write_to_zincrc": "Would you like to write the information you just entered to a configuration file (.zincrc) so you can make orders more easily in the future? We'll only include your shipping address and a hashed credit card token, so no confidential information will be written to your hard drive. (y)/n"
+        "write_to_zincrc": "Would you like to write the information you just entered to a configuration file (~/.zincrc) so you can make orders more easily in the future? We'll only include your shipping address and a hashed credit card token, so no confidential information will be written to your hard drive. [y]/n"
         }
 
     def __init__(self,
@@ -135,7 +131,7 @@ class ZincWizard(object):
         self.shipping_address = None
 
     def get_stored_data(self, filename):
-        default_filename = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../.zincrc")
+        default_filename = os.path.join(os.path.expanduser("~"), ".zincrc")
         if filename != None and os.path.isfile(filename):
             with open(filename, 'rb') as f:
                 return json.loads(f.read())
@@ -192,9 +188,11 @@ class ZincWizard(object):
                     print "You've reached the maximum number of attempts. Exiting!"
                     sys.exit()
 
-    def prompt_boolean(self, prompt):
+    def prompt_boolean(self, prompt, default=True):
         result = self.prompt(prompt, ValidationHelpers.validate_boolean()).strip()
-        if (result == "y" or result == ""):
+        if (result == ""):
+            return default
+        elif (result == "y"):
             return True
         return False
 
@@ -220,7 +218,8 @@ class ZincWizard(object):
         asin = self.get_asin(self.product_url)
         product_info = None
         if asin != None:
-            print call(["curl", "http://ihmage.com/" + asin + "?size=25"])
+            InterfaceHelpers.print_ihmage_image(asin)
+            print "\nLoading product information...\n"
             product_info = AmazonDataFinder.get_amazon_data(asin)
         variants_response = async_response.get_response()
         response_data["variant_options_response"] = variants_response
@@ -318,7 +317,7 @@ class ZincWizard(object):
         self.print_indent("Total:            %s" % format_price(components["total"]))
 
     def write_to_zincrc(self):
-        filename = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../.zincrc")
+        filename = os.path.join(os.path.expanduser("~"), ".zincrc")
         if not os.path.isfile(filename) and self.prompt_boolean(self.PROMPTS["write_to_zincrc"]):
             data = {
                 "shipping_address": self.shipping_address,
@@ -360,12 +359,18 @@ class ZincWizard(object):
 
     def get_address(self, filetype):
         address = {}
-        for label in ["first_name", "last_name", "address_line1", "address_line2",
+        for label in ["name", "address_line1", "address_line2",
                 "city", "state", "zip_code", "country"]:
-            address[label] = self.prompt(self.PROMPTS["address"][label])
+            if label == "name":
+                full_name = self.prompt(self.PROMPTS["address"][label])
+                (first_name, last_name) = self.parse_name(full_name)
+                address["first_name"] = first_name
+                address["last_name"] = last_name
+            else:
+                address[label] = self.prompt(self.PROMPTS["address"][label])
 
         print "\nYou typed the following:\n"
-        self.print_indent("    %s %s" % (address["first_name"], address["last_name"]))
+        self.print_indent("    %s %s" % (address["name"]))
         self.print_indent("    %s" % (address["address_line1"]))
         self.print_indent("    %s" % (address["address_line2"]))
         self.print_indent("    %s, %s %s" % (address["city"], address["state"], address["zip_code"]))
@@ -378,6 +383,15 @@ class ZincWizard(object):
         else:
             return self.get_address(filetype)
 
+    def parse_name(self, full_name):
+        split = full_name.split(" ")
+        if len(split) > 1:
+            first_name = " ".join(split[:(-1)])
+            last_name = split[-1]
+            return (first_name, last_name)
+        else:
+            return (".", full_name)
+
     def build_prompt(self, base_prompt, description_list):
         prompt = base_prompt + "\n"
         prompt += "\n".join(description_list)
@@ -395,7 +409,10 @@ class ZincWizard(object):
         collector = []
         for i in xrange(len(response["results"])):
             current = response["results"][i]
-            descriptions.append(str(i) + ") " + current["title"])
+            new_description = str(i) + ") " + current["title"]
+            if "price" in current:
+                new_description += ", " + format_price(current["price"])
+            descriptions.append(new_description)
             asin = self.get_asin(current["product_url"])
             collector.append(current["product_url"])
 
@@ -490,5 +507,39 @@ class AmazonDataFinder(object):
         except Exception, err:
             return False
 
+class InterfaceHelpers(object):
+    MAX_IHMAGE_WIDTH = 50
+
+    @classmethod
+    def get_terminal_size(klass):
+        import os
+        env = os.environ
+        def ioctl_GWINSZ(fd):
+            try:
+                import fcntl, termios, struct, os
+                cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,
+            '1234'))
+            except:
+                return
+            return cr
+        cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
+        if not cr:
+            try:
+                fd = os.open(os.ctermid(), os.O_RDONLY)
+                cr = ioctl_GWINSZ(fd)
+                os.close(fd)
+            except:
+                pass
+        if not cr:
+            ### Use get(key[, default]) instead of a try/catch
+            cr = (env.get('LINES', 25), env.get('COLUMNS', 80))
+        return int(cr[1]), int(cr[0])
+
+    @classmethod
+    def print_ihmage_image(klass, asin):
+        term_width = InterfaceHelpers.get_terminal_size()[0]
+        size = min(term_width/3, InterfaceHelpers.MAX_IHMAGE_WIDTH)
+        call(["curl", "http://ihmage.com/" + asin + "?size="+str(size)])
+
 if __name__ == '__main__':
     ZincWizard().start()
